from django.template import RequestContext, loader
from django.template.base import TemplateDoesNotExist
from django.views.decorators.cache import cache_control
from django.views.generic import TemplateView

from bulbs.content.models import Content
from bulbs.content.views import BaseContentDetailView
from bulbs.feeds.views import RSSView


class InstantArticleRSSView(RSSView):
    paginate_by = 100
    template_name = "feeds/instant_article_rss.xml"
    feed_title = "Instant Articles RSS Feed"

    def get_queryset(self):
        return Content.search_objects.instant_articles()

    def get_template_names(self):
        return [
            "feeds/instant_article_rss.xml",
            "instant_article/base_instant_article_rss.xml"
        ]

    def get_context_data(self, *args, **kwargs):
        context = super(InstantArticleRSSView, self).get_context_data(*args, **kwargs)
        context["title"] = self.feed_title
        context["search_url"] = self.request.build_absolute_uri("/")

        for content in context["page_obj"].object_list:
            content.feed_url = self.request.build_absolute_uri(content.get_absolute_url())
            content_ctx = {
                "content": content,
                "absolute_uri": self.request.META.get('HTTP_HOST', None)
            }
            try:
                content.instant_article_html = loader.render_to_string(
                    "instant_article/_instant_article.html", content_ctx
                )
            except TemplateDoesNotExist:
                content.instant_article_html = loader.render_to_string(
                    "instant_article/base_instant_article.html", content_ctx
                )

        return RequestContext(self.request, context)


class InstantArticleContentView(BaseContentDetailView):

    redirect_correct_path = False

    def get_template_names(self):
        return [
            "instant_article/_instant_article.html",
            "instant_article/base_instant_article.html"
        ]

    def get_context_data(self, *args, **kwargs):
        context = super(InstantArticleContentView, self).get_context_data(*args, **kwargs)
        targeting = self.object.get_targeting()
        context["targeting"] = targeting
        context["absolute_uri"] = self.request.META.get("HTTP_HOST", None)
        return context


class InstantArticleAdView(InstantArticleContentView):
    def get_template_names(self):
        return [
            "instant_article/_instant_article_ad.html",
        ]

class InstantArticleAnalyticsView(InstantArticleContentView):
    def get_template_names(self):
        return [
            "core/_analytics.html",
        ]

    def get_context_data(self, *args, **kwargs):
        context = super(InstantArticleAnalyticsView, self).get_context_data(*args, **kwargs)
        context["fire_pageview"] = True
        context["platform"] = "Instant Articles"
        context["path"] = self.request.GET.get("path", "")
        return context


instant_article_rss = cache_control(max_age=600)(InstantArticleRSSView.as_view())
instant_article = cache_control(max_age=600)(InstantArticleContentView.as_view())
instant_article_analytics = cache_control(max_age=600)(InstantArticleAnalyticsView.as_view())
instant_article_ad = cache_control(max_age=600)(InstantArticleAdView.as_view())



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer
from bulbs.utils.serializers import JSONField

from .models import Section


class SectionSerializer(serializers.ModelSerializer):

    query = JSONField(required=False, default={})
    section_logo = ImageFieldSerializer(required=False, allow_null=True)

    class Meta:
        model = Section



from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.template.defaultfilters import slugify
from djbetty.serializers import ImageFieldSerializer
from rest_framework.utils import model_meta
from rest_framework import relations
from rest_framework import serializers
from six import string_types

from .models import Content, Tag, LogEntry, FeatureType, TemplateType, ObfuscatedUrlInfo


class ContentTypeField(serializers.Field):
    """Converts between natural key for native use and integer for non-native."""
    def to_representation(self, value):
        """Convert to natural key."""
        content_type = ContentType.objects.get_for_id(value)
        return "_".join(content_type.natural_key())

    def to_internal_value(self, value):
        """Convert to integer id."""
        natural_key = value.split("_")
        content_type = ContentType.objects.get_by_natural_key(*natural_key)
        return content_type.id


class PolymorphicSerializerMixin(object):
    """Serialize a mix of polymorphic models with their own serializer classes."""
    def to_representation(self, value):
        if value:
            if hasattr(value, "get_serializer_class"):
                ThisSerializer = value.get_serializer_class()
            else:
                class ThisSerializer(serializers.ModelSerializer):
                    class Meta:
                        model = value.__class__

            serializer = ThisSerializer(context=self.context)
            return serializer.to_representation(value)
        else:
            return super(PolymorphicSerializerMixin, self).to_representation(value)


class TagSerializer(serializers.ModelSerializer):
    name = serializers.CharField(max_length=50)  # this is actually limited by the slug

    class Meta:
        model = Tag

    def to_representation(self, obj):
        doc_type = obj.get_real_instance().mapping.doc_type
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
            "type": doc_type
        }


class FeatureTypeSerializer(serializers.ModelSerializer):

    class Meta:
        model = FeatureType

    def to_representation(self, obj):
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
        }


class TemplateTypeSerializer(serializers.ModelSerializer):

    class Meta:
        model = TemplateType

    def to_representation(self, obj):
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
            "content_type": obj.content_type
        }


class TagField(relations.RelatedField):
    """This is a relational field that handles the addition of tags to content
    objects. This field also allows the user to create tags in the db if they
    don't already exist."""

    read_only = False

    def to_representation(self, obj):
        return {
            "id": obj.pk,
            "name": obj.name,
            "slug": obj.slug,
            "type": obj.__class__.search_objects.mapping.doc_type
        }

    def to_internal_value(self, value):
        """Basically, each tag dict must include a full dict with id,
        name and slug--or else you need to pass in a dict with just a name,
        which indicated that the Tag doesn't exist, and should be added."""

        if "id" in value:
            tag = Tag.objects.get(id=value["id"])
        else:
            if "name" not in value:
                raise ValidationError("Tags must include an ID or a name.")
            if len(slugify(value["name"])) > 50:
                raise ValidationError("Maximum tag length is 50 characters.")

            name = value["name"]
            slug = value.get("slug", slugify(name))
            try:
                tag = Tag.objects.get(slug=slug)
            except Tag.DoesNotExist:
                tag, created = Tag.objects.get_or_create(name=name, slug=slug)
        return tag


class FeatureTypeField(relations.RelatedField):
    """This is a relational field that handles the addition of feature_types to
    content objects. This field also allows the user to create feature_types in
    the db if they don't already exist."""

    read_only = False

    def to_representation(self, obj):
        return obj.name

    def to_internal_value(self, value):
        """Basically, each tag dict must include a full dict with id,
        name and slug--or else you need to pass in a dict with just a name,
        which indicated that the FeatureType doesn't exist, and should be added."""
        if value == "":
            return None

        if isinstance(value, string_types):
            slug = slugify(value)
            feature_type, created = FeatureType.objects.get_or_create(
                slug=slug,
                defaults={"name": value}
            )
        else:
            if "id" in value:
                feature_type = FeatureType.objects.get(id=value["id"])
            elif "slug" in value:
                feature_type = FeatureType.objects.get(slug=value["slug"])
            else:
                raise ValidationError("Invalid feature type data")
        return feature_type


class DefaultUserSerializer(serializers.ModelSerializer):
    """Returns basic User fields"""

    class Meta:
        model = get_user_model()

    def to_representation(self, obj):

        json = {
            "id": obj.pk,
            "username": obj.username,
            "email": obj.email,
            "first_name": obj.first_name,
            "last_name": obj.last_name,
            "full_name": obj.get_full_name(),
            "short_name": obj.get_short_name()
        }

        if hasattr(obj, 'freelanceprofile'):
            json['is_manager'] = obj.freelanceprofile.is_manager

        return json

    def to_internal_value(self, data):
        """Basically, each author dict must include either a username or id."""
        # model = get_user_model()
        model = self.Meta.model

        if "id" in data:
            author = model.objects.get(id=data["id"])
        else:
            if "username" not in data:
                raise ValidationError("Authors must include an ID or a username.")
            username = data["username"]
            author = model.objects.get(username=username)
        return author


UserSerializer = getattr(settings, "BULBS_USER_SERIALIZER", DefaultUserSerializer)
AUTHOR_FILTER = getattr(settings, "BULBS_AUTHOR_FILTER", {"is_staff": True})


class AuthorField(relations.RelatedField):
    """This field handles the addition/removal of authors to content"""

    read_only = False

    def to_representation(self, obj):
        return {
            "id": obj.pk,
            "username": obj.username,
            "email": obj.email,
            "first_name": obj.first_name,
            "last_name": obj.last_name,
            "full_name": obj.get_full_name(),
            "short_name": obj.get_short_name()
        }

    def to_internal_value(self, data):
        """Basically, each author dict must include either a username or id."""
        # model = get_user_model()
        model = get_user_model()

        if data is None:
            return None

        if "id" in data:
            author = model.objects.get(id=data["id"])
        elif "username" in data:
            author = model.objects.get(username=data["username"])
        else:
            raise ValidationError("Authors must include an ID or a username.")
        return author


class ContentSerializer(serializers.ModelSerializer):

    polymorphic_ctype = ContentTypeField(source="polymorphic_ctype_id", read_only=True)
    tags = TagField(allow_null=True, many=True, queryset=Tag.objects.all(), required=False)
    feature_type = FeatureTypeField(allow_null=True, queryset=FeatureType.objects.all(), required=False)
    authors = AuthorField(many=True, allow_null=True, queryset=get_user_model().objects.filter(**AUTHOR_FILTER), required=False)
    thumbnail = ImageFieldSerializer(allow_null=True, read_only=True)
    first_image = ImageFieldSerializer(allow_null=True, read_only=True)
    thumbnail_override = ImageFieldSerializer(allow_null=True, required=False)
    absolute_url = serializers.ReadOnlyField(source="get_absolute_url")
    status = serializers.ReadOnlyField(source="get_status")
    template_type = serializers.SlugRelatedField(
        slug_field="slug",
        allow_null=True,
        queryset=TemplateType.objects.all(),
        required=False)

    class Meta:
        model = Content

    def create(self, validated_data):
        ModelClass = self.Meta.model

        # Remove many-to-many relationships from validated_data.
        # They are not valid arguments to the default `.create()` method,
        # as they require that the instance has already been saved.
        info = model_meta.get_field_info(ModelClass)
        many_to_many = {}
        for field_name, relation_info in info.relations.items():
            if relation_info.to_many and (field_name in validated_data):
                many_to_many[field_name] = validated_data.pop(field_name)

        try:
            instance = ModelClass.objects.create(**validated_data)
        except TypeError as exc:
            msg = (
                'Got a `TypeError` when calling `%s.objects.create()`. '
                'This may be because you have a writable field on the '
                'serializer class that is not a valid argument to '
                '`%s.objects.create()`. You may need to make the field '
                'read-only, or override the %s.create() method to handle '
                'this correctly.\nOriginal exception text was: %s.' %
                (
                    ModelClass.__name__,
                    ModelClass.__name__,
                    self.__class__.__name__,
                    exc
                )
            )
            raise TypeError(msg)

        # Save many-to-many relationships after the instance is created.
        if many_to_many:
            for field_name, value in many_to_many.items():
                setattr(instance, field_name, value)

        return instance


class LogEntrySerializer(serializers.ModelSerializer):

    class Meta:
        model = LogEntry


class PolymorphicContentSerializer(PolymorphicSerializerMixin, ContentSerializer):
    pass


class ObfuscatedUrlInfoSerializer(serializers.ModelSerializer):

    content = serializers.PrimaryKeyRelatedField(queryset=Content.objects.all())
    expire_date = serializers.DateTimeField()
    create_date = serializers.DateTimeField()
    url_uuid = serializers.CharField(min_length=32, max_length=32, required=False)

    def validate(self, value):
        super(ObfuscatedUrlInfoSerializer, self).validate(value)
        if value["expire_date"] < value["create_date"]:
            raise serializers.ValidationError(
                "Start date must occur before expiration date.")
        return value

    class Meta:
        model = ObfuscatedUrlInfo



from rest_framework import viewsets
from rest_framework import filters
from rest_framework.permissions import IsAdminUser
from rest_framework.response import Response

from bulbs.api.permissions import CanEditContent
from bulbs.utils.methods import get_query_params
from .models import Poll, Answer
from .serializers import PollSerializer, AnswerSerializer


class PollViewSet(viewsets.ModelViewSet):
    model = Poll
    queryset = Poll.objects.all()
    permission_classes = [
        IsAdminUser,
        CanEditContent,
    ]
    serializer_class = PollSerializer
    paginate_by = 20
    filter_backends = (
            filters.OrderingFilter,
            filters.SearchFilter,)
    ordering_fields = (
            "title",
            "published",
            "end_date",)
    search_fields = [
            "answers",
            "title",]

    class Meta:
        exclude = ('thumbnail_override',)

    def list(self, request, *args, **kwargs):
        """Modified list view to driving listing from ES"""
        search_kwargs = {"published": False}

        query_params = get_query_params(self.request)

        for field_name in ("status"):
            if field_name in query_params:
                search_kwargs[field_name] = query_params.get(field_name)

        if "search" in query_params:
            search_kwargs["query"] = query_params.get("search")

        # active/closed are semantics of the Poll model
        # active is analagous to 'published' in Content
        # closed is in relation to the end_date of a Poll
        if "active" in query_params:
            del search_kwargs["published"]
            search_kwargs["active"] = True

        if "closed" in query_params:
            del search_kwargs["published"]
            search_kwargs["closed"] = True

        queryset = self.model.search_objects.search(**search_kwargs)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class AnswerViewSet(viewsets.ModelViewSet):
    model = Answer
    queryset = Answer.objects.all()
    permission_classes = [
        IsAdminUser,
        CanEditContent,
    ]
    serializer_class = AnswerSerializer



"""celery tasks for contributions."""
from celery import shared_task

from .email import EmailReport
from .models import Contribution


@shared_task(default_retry_delay=5)
def update_role_rates(contributor_role_pk):
    for contribution in Contribution.objects.filter(contributor__pk=contributor_role_pk):
        contribution.index()


@shared_task(default_retry_delay=5)
def run_contributor_email_report(**kwargs):
    report = EmailReport(**kwargs)
    report.send_mass_contributor_emails()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('content', '0003_auto_20150513_2326'),
    ]

    operations = [
        migrations.AlterField(
            model_name='content',
            name='description',
            field=models.TextField(blank=True, default='', max_length=1024),
        ),
        migrations.AlterField(
            model_name='content',
            name='slug',
            field=models.SlugField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='content',
            name='subhead',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='action_time',
            field=models.DateTimeField(verbose_name='action time', auto_now=True),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='change_message',
            field=models.TextField(verbose_name='change message', blank=True),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='object_id',
            field=models.TextField(verbose_name='object id', blank=True, null=True),
        ),
    ]
